@@ -1,14 +1,18 @@
 use crate::{
-    builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
+    custom::KeyBindingsConfig,
     markdown::parse::ParseError,
+    media::{
+        image::{Image, ImageSource},
+        printer::{ImageResource, ResourceProperties},
+    },
     presentation::{Presentation, RenderOperation},
-    render::media::{Image, ImageSource},
+    processing::builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
     tools::{ExecutionError, ThirdPartyTools},
     typst::TypstRender,
     CodeHighlighter, MarkdownParser, PresentationTheme, Resources,
 };
 use base64::{engine::general_purpose::STANDARD, Engine};
-use image::{codecs::png::PngEncoder, DynamicImage, GenericImageView, ImageEncoder, ImageError};
+use image::{codecs::png::PngEncoder, DynamicImage, ImageEncoder, ImageError};
 use semver::Version;
 use serde::Serialize;
 use std::{
@@ -45,11 +49,11 @@ impl<'a> Exporter<'a> {
     /// Export the given presentation into PDF.
     ///
     /// This uses a separate `presenterm-export` tool.
-    pub fn export_pdf(&mut self, presentation_path: &Path) -> Result<(), ExportError> {
+    pub fn export_pdf(&mut self, presentation_path: &Path, extra_args: &[&str]) -> Result<(), ExportError> {
         Self::validate_exporter_version()?;
 
         let metadata = self.generate_metadata(presentation_path)?;
-        Self::execute_exporter(metadata)?;
+        Self::execute_exporter(metadata, extra_args)?;
         Ok(())
     }
 
@@ -81,6 +85,8 @@ impl<'a> Exporter<'a> {
             &mut self.resources,
             &mut self.typst,
             &self.themes,
+            Default::default(),
+            KeyBindingsConfig::default(),
             self.options.clone(),
         )
         .build(elements)?;
@@ -92,14 +98,15 @@ impl<'a> Exporter<'a> {
         Ok(metadata)
     }
 
-    fn execute_exporter(metadata: ExportMetadata) -> Result<(), ExportError> {
+    fn execute_exporter(metadata: ExportMetadata, extra_args: &[&str]) -> Result<(), ExportError> {
         let presenterm_path = env::current_exe().map_err(ExportError::Io)?;
         let presenterm_path = presenterm_path.display().to_string();
         let presentation_path = metadata.presentation_path.display().to_string();
         let metadata = serde_json::to_vec(&metadata).expect("serialization failed");
-        ThirdPartyTools::presenterm_export(&[&presenterm_path, "--export", &presentation_path])
-            .stdin(metadata)
-            .run()?;
+        let mut args = vec![&presenterm_path, "--export"];
+        args.extend(extra_args);
+        args.push(&presentation_path);
+        ThirdPartyTools::presenterm_export(&args).stdin(metadata).run()?;
         Ok(())
     }
 
@@ -109,7 +116,7 @@ impl<'a> Exporter<'a> {
         let mut next_slide = |commands: &mut Vec<CaptureCommand>| {
             commands.push(CaptureCommand::SendKeys { keys: "l" });
             commands.push(CaptureCommand::WaitForChange);
-            presentation.jump_next_slide();
+            presentation.jump_next();
         };
         for chunks in slide_chunks {
             for _ in 0..chunks - 1 {
@@ -135,11 +142,14 @@ impl<'a> Exporter<'a> {
                 ImageSource::Generated => {
                     let mut buffer = Vec::new();
                     let dimensions = image.original.dimensions();
+                    let ImageResource::Ascii(resource) = image.original.resource.as_ref() else {
+                        panic!("not in viuer mode")
+                    };
                     PngEncoder::new(&mut buffer).write_image(
-                        image.original.as_bytes(),
+                        resource.as_bytes(),
                         dimensions.0,
                         dimensions.1,
-                        image.original.color(),
+                        resource.color(),
                     )?;
                     let contents = Some(STANDARD.encode(buffer));
                     ImageMetadata { path: None, color: image.color, contents }
@@ -236,11 +246,11 @@ pub(crate) struct ImageReplacer {
 impl ImageReplacer {
     pub(crate) fn replace_presentation_images(&mut self, presentation: &mut Presentation) {
         let callback = |operation: &mut RenderOperation| {
-            let RenderOperation::RenderImage(image) = operation else {
+            let RenderOperation::RenderImage(image, properties) = operation else {
                 return;
             };
             let replacement = self.replace_image(image.clone());
-            *operation = RenderOperation::RenderImage(replacement);
+            *operation = RenderOperation::RenderImage(replacement, properties.clone());
         };
 
         presentation.mutate_operations(callback);
@@ -257,7 +267,8 @@ impl ImageReplacer {
             pixel.0 = rgb_color;
         }
         self.images.push(ReplacedImage { original: image, color });
-        Image::new(replacement)
+
+        Image::new(ImageResource::Ascii(replacement.into()), ImageSource::Generated)
     }
 
     fn allocate_color(&mut self) -> u32 {
@@ -287,7 +298,7 @@ mod test {
         let arena = Arena::new();
         let parser = MarkdownParser::new(&arena);
         let theme = PresentationThemeSet::default().load_by_name("dark").unwrap();
-        let resources = Resources::new("examples");
+        let resources = Resources::new("examples", Default::default());
         let typst = TypstRender::default();
         let themes = Themes::default();
         let options = PresentationBuilderOptions { allow_mutations: false, ..Default::default() };
