@@ -2,15 +2,16 @@
 //!
 //! This is not meant to be used as a crate!
 
-pub(crate) mod builder;
 pub(crate) mod custom;
 pub(crate) mod diff;
 pub(crate) mod execute;
 pub(crate) mod export;
 pub(crate) mod input;
 pub(crate) mod markdown;
+pub(crate) mod media;
 pub(crate) mod presentation;
 pub(crate) mod presenter;
+pub(crate) mod processing;
 pub(crate) mod render;
 pub(crate) mod resource;
 pub(crate) mod style;
@@ -19,16 +20,14 @@ pub(crate) mod tools;
 pub(crate) mod typst;
 
 pub use crate::{
-    builder::{PresentationBuilderOptions, Themes},
-    custom::Config,
+    custom::{Config, ImageProtocol},
     export::{ExportError, Exporter},
     input::source::CommandSource,
     markdown::parse::MarkdownParser,
-    presenter::{PresentMode, Presenter},
-    render::{
-        highlighting::{CodeHighlighter, HighlightThemeSet},
-        media::MediaRender,
-    },
+    media::{graphics::GraphicsMode, printer::ImagePrinter, register::ImageRegistry},
+    presenter::{PresentMode, Presenter, PresenterOptions},
+    processing::builder::{PresentationBuilderOptions, Themes},
+    render::highlighting::{CodeHighlighter, HighlightThemeSet},
     resource::Resources,
     theme::{LoadThemeError, PresentationTheme, PresentationThemeSet},
     typst::TypstRender,
