@@ -1,24 +1,32 @@
 use crate::{
     custom::OptionsConfig,
-    markdown::text::WeightedLine,
-    render::{media::Image, properties::WindowSize},
+    markdown::text::WeightedTextBlock,
+    media::image::Image,
+    render::properties::WindowSize,
     style::Colors,
     theme::{Alignment, Margin, PresentationTheme},
 };
 use serde::Deserialize;
-use std::{fmt::Debug, rc::Rc};
+use std::{cell::RefCell, fmt::Debug, ops::Deref, rc::Rc};
+
+#[derive(Debug)]
+pub(crate) struct Modals {
+    pub(crate) slide_index: Vec<RenderOperation>,
+    pub(crate) bindings: Vec<RenderOperation>,
+}
 
 /// A presentation.
 #[derive(Debug)]
 pub(crate) struct Presentation {
     slides: Vec<Slide>,
-    current_slide_index: usize,
+    modals: Modals,
+    state: PresentationState,
 }
 
 impl Presentation {
     /// Construct a new presentation.
-    pub(crate) fn new(slides: Vec<Slide>) -> Self {
-        Self { slides, current_slide_index: 0 }
+    pub(crate) fn new(slides: Vec<Slide>, modals: Modals, state: PresentationState) -> Self {
+        Self { slides, modals, state }
     }
 
     /// Iterate the slides in this presentation.
@@ -26,6 +34,16 @@ impl Presentation {
         self.slides.iter()
     }
 
+    /// Iterate the operations that render the slide index.
+    pub(crate) fn iter_slide_index_operations(&self) -> impl Iterator<Item = &RenderOperation> {
+        self.modals.slide_index.iter()
+    }
+
+    /// Iterate the operations that render the key bindings modal.
+    pub(crate) fn iter_bindings_operations(&self) -> impl Iterator<Item = &RenderOperation> {
+        self.modals.bindings.iter()
+    }
+
     /// Consume this presentation and return its slides.
     #[cfg(test)]
     pub(crate) fn into_slides(self) -> Vec<Slide> {
@@ -34,22 +52,23 @@ impl Presentation {
 
     /// Get the current slide.
     pub(crate) fn current_slide(&self) -> &Slide {
-        &self.slides[self.current_slide_index]
+        &self.slides[self.current_slide_index()]
     }
 
     /// Get the current slide index.
     pub(crate) fn current_slide_index(&self) -> usize {
-        self.current_slide_index
+        self.state.current_slide_index()
     }
 
-    /// Jump to the next slide.
-    pub(crate) fn jump_next_slide(&mut self) -> bool {
+    /// Jump forwards.
+    pub(crate) fn jump_next(&mut self) -> bool {
         let current_slide = self.current_slide_mut();
         if current_slide.move_next() {
             return true;
         }
-        if self.current_slide_index < self.slides.len() - 1 {
-            self.current_slide_index += 1;
+        let current_slide_index = self.current_slide_index();
+        if current_slide_index < self.slides.len() - 1 {
+            self.state.set_current_slide_index(current_slide_index + 1);
             // Going forward we show only the first chunk.
             self.current_slide_mut().show_first_chunk();
             true
@@ -58,14 +77,15 @@ impl Presentation {
         }
     }
 
-    /// Jump to the previous slide.
-    pub(crate) fn jump_previous_slide(&mut self) -> bool {
+    /// Jump backwards.
+    pub(crate) fn jump_previous(&mut self) -> bool {
         let current_slide = self.current_slide_mut();
         if current_slide.move_previous() {
             return true;
         }
-        if self.current_slide_index > 0 {
-            self.current_slide_index -= 1;
+        let current_slide_index = self.current_slide_index();
+        if current_slide_index > 0 {
+            self.state.set_current_slide_index(current_slide_index - 1);
             // Going backwards we show all chunks.
             self.current_slide_mut().show_all_chunks();
             true
@@ -76,19 +96,19 @@ impl Presentation {
 
     /// Jump to the first slide.
     pub(crate) fn jump_first_slide(&mut self) -> bool {
-        self.jump_slide(0)
+        self.go_to_slide(0)
     }
 
     /// Jump to the last slide.
     pub(crate) fn jump_last_slide(&mut self) -> bool {
         let last_slide_index = self.slides.len().saturating_sub(1);
-        self.jump_slide(last_slide_index)
+        self.go_to_slide(last_slide_index)
     }
 
     /// Jump to a specific slide.
-    pub(crate) fn jump_slide(&mut self, slide_index: usize) -> bool {
+    pub(crate) fn go_to_slide(&mut self, slide_index: usize) -> bool {
         if slide_index < self.slides.len() {
-            self.current_slide_index = slide_index;
+            self.state.set_current_slide_index(slide_index);
             // Always show only the first slide when jumping to a particular one.
             self.current_slide_mut().show_first_chunk();
             true
@@ -148,7 +168,58 @@ impl Presentation {
     }
 
     fn current_slide_mut(&mut self) -> &mut Slide {
-        &mut self.slides[self.current_slide_index]
+        let index = self.current_slide_index();
+        &mut self.slides[index]
+    }
+}
+
+impl From<Vec<Slide>> for Presentation {
+    fn from(slides: Vec<Slide>) -> Self {
+        let modals = Modals { slide_index: vec![], bindings: vec![] };
+        Self::new(slides, modals, Default::default())
+    }
+}
+
+#[derive(Debug, Default)]
+pub(crate) struct PresentationStateInner {
+    current_slide_index: usize,
+}
+
+#[derive(Clone, Debug, Default)]
+pub(crate) struct PresentationState {
+    inner: Rc<RefCell<PresentationStateInner>>,
+}
+
+impl PresentationState {
+    pub(crate) fn current_slide_index(&self) -> usize {
+        self.inner.deref().borrow().current_slide_index
+    }
+
+    fn set_current_slide_index(&self, value: usize) {
+        self.inner.deref().borrow_mut().current_slide_index = value;
+    }
+}
+
+/// A slide builder.
+#[derive(Default)]
+pub(crate) struct SlideBuilder {
+    chunks: Vec<SlideChunk>,
+    footer: Vec<RenderOperation>,
+}
+
+impl SlideBuilder {
+    pub(crate) fn chunks(mut self, chunks: Vec<SlideChunk>) -> Self {
+        self.chunks = chunks;
+        self
+    }
+
+    pub(crate) fn footer(mut self, footer: Vec<RenderOperation>) -> Self {
+        self.footer = footer;
+        self
+    }
+
+    pub(crate) fn build(self) -> Slide {
+        Slide::new(self.chunks, self.footer)
     }
 }
 
@@ -372,19 +443,24 @@ pub(crate) enum RenderOperation {
     /// Jump the draw cursor into the vertical center, that is, at `screen_height / 2`.
     JumpToVerticalCenter,
 
-    /// Jumps to the N-th to last row in the slide.
+    /// Jumps to the N-th row in the current layout.
+    ///
+    /// The index is zero based where 0 represents the top row.
+    JumpToRow { index: u16 },
+
+    /// Jumps to the N-th to last row in the current layout.
     ///
     /// The index is zero based where 0 represents the bottom row.
     JumpToBottomRow { index: u16 },
 
     /// Render text.
-    RenderText { line: WeightedLine, alignment: Alignment },
+    RenderText { line: WeightedTextBlock, alignment: Alignment },
 
     /// Render a line break.
     RenderLineBreak,
 
     /// Render an image.
-    RenderImage(Image),
+    RenderImage(Image, ImageProperties),
 
     /// Render a preformatted line.
     ///
@@ -423,6 +499,22 @@ pub(crate) enum RenderOperation {
     PopMargin,
 }
 
+/// The properties of an image being rendered.
+#[derive(Clone, Debug, Default)]
+pub(crate) struct ImageProperties {
+    pub(crate) z_index: i32,
+    pub(crate) size: ImageSize,
+    pub(crate) restore_cursor: bool,
+}
+
+/// The size used when printing an image.
+#[derive(Clone, Debug, Default)]
+pub(crate) enum ImageSize {
+    #[default]
+    Scaled,
+    Specific(u16, u16),
+}
+
 /// Slide properties, set on initialization.
 #[derive(Clone, Debug, Default)]
 pub(crate) struct MarginProperties {
@@ -482,9 +574,9 @@ mod test {
             match self {
                 First => presentation.jump_first_slide(),
                 Last => presentation.jump_last_slide(),
-                Next => presentation.jump_next_slide(),
-                Previous => presentation.jump_previous_slide(),
-                Specific(index) => presentation.jump_slide(*index),
+                Next => presentation.jump_next(),
+                Previous => presentation.jump_previous(),
+                Specific(index) => presentation.go_to_slide(*index),
             };
         }
 
@@ -555,12 +647,12 @@ mod test {
         #[case] expected_slide: usize,
         #[case] expected_chunk: usize,
     ) {
-        let mut presentation = Presentation::new(vec![
+        let mut presentation = Presentation::from(vec![
             Slide::new(vec![SlideChunk::from(SlideChunk::default()), SlideChunk::default()], vec![]),
             Slide::new(vec![SlideChunk::from(SlideChunk::default()), SlideChunk::default()], vec![]),
             Slide::new(vec![SlideChunk::from(SlideChunk::default()), SlideChunk::default()], vec![]),
         ]);
-        presentation.jump_slide(from);
+        presentation.go_to_slide(from);
 
         for jump in jumps {
             jump.apply(&mut presentation);
@@ -592,26 +684,24 @@ mod test {
         #[case] expected_slide: usize,
         #[case] expected_chunk: usize,
     ) {
-        let mut presentation = Presentation::new(vec![
-            Slide::new(
-                vec![
+        let mut presentation = Presentation::from(vec![
+            SlideBuilder::default()
+                .chunks(vec![
                     SlideChunk::from(SlideChunk::new(
                         vec![],
                         vec![Box::new(DummyMutator::new(1)), Box::new(DummyMutator::new(2))],
                     )),
                     SlideChunk::default(),
-                ],
-                vec![],
-            ),
-            Slide::new(
-                vec![
+                ])
+                .build(),
+            SlideBuilder::default()
+                .chunks(vec![
                     SlideChunk::from(SlideChunk::new(vec![], vec![Box::new(DummyMutator::new(2))])),
                     SlideChunk::default(),
-                ],
-                vec![],
-            ),
+                ])
+                .build(),
         ]);
-        presentation.jump_slide(from);
+        presentation.go_to_slide(from);
 
         for jump in jumps {
             jump.apply(&mut presentation);
