@@ -1,14 +1,15 @@
 use crate::{
-    builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
+    custom::KeyBindingsConfig,
     diff::PresentationDiffer,
     export::ImageReplacer,
     input::source::{Command, CommandSource},
     markdown::parse::{MarkdownParser, ParseError},
+    media::{printer::ImagePrinter, register::ImageRegistry},
     presentation::Presentation,
+    processing::builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
     render::{
         draw::{RenderError, RenderResult, TerminalDrawer},
         highlighting::CodeHighlighter,
-        media::GraphicsMode,
     },
     resource::Resources,
     theme::PresentationTheme,
@@ -20,8 +21,16 @@ use std::{
     io::{self, Stdout},
     mem,
     path::Path,
+    rc::Rc,
 };
 
+pub struct PresenterOptions {
+    pub mode: PresentMode,
+    pub builder_options: PresentationBuilderOptions,
+    pub font_size_fallback: u8,
+    pub bindings: KeyBindingsConfig,
+}
+
 /// A slideshow presenter.
 ///
 /// This type puts everything else together.
@@ -31,11 +40,11 @@ pub struct Presenter<'a> {
     parser: MarkdownParser<'a>,
     resources: Resources,
     typst: TypstRender,
-    mode: PresentMode,
     state: PresenterState,
     slides_with_pending_widgets: HashSet<usize>,
+    image_printer: Rc<ImagePrinter>,
     themes: Themes,
-    builder_options: PresentationBuilderOptions,
+    options: PresenterOptions,
 }
 
 impl<'a> Presenter<'a> {
@@ -48,8 +57,8 @@ impl<'a> Presenter<'a> {
         resources: Resources,
         typst: TypstRender,
         themes: Themes,
-        mode: PresentMode,
-        builder_options: PresentationBuilderOptions,
+        image_printer: Rc<ImagePrinter>,
+        options: PresenterOptions,
     ) -> Self {
         Self {
             default_theme,
@@ -57,23 +66,21 @@ impl<'a> Presenter<'a> {
             parser,
             resources,
             typst,
-            mode,
             state: PresenterState::Empty,
             slides_with_pending_widgets: HashSet::new(),
+            image_printer,
             themes,
-            builder_options,
+            options,
         }
     }
 
     /// Run a presentation.
     pub fn present(mut self, path: &Path) -> Result<(), PresentationError> {
-        self.state = PresenterState::Presenting(self.load_presentation(path)?);
+        self.state = PresenterState::Presenting(Presentation::from(vec![]));
+        self.try_reload(path);
 
-        let graphics_mode = match self.mode {
-            PresentMode::Export => GraphicsMode::AsciiBlocks,
-            _ => GraphicsMode::default(),
-        };
-        let mut drawer = TerminalDrawer::new(io::stdout(), graphics_mode)?;
+        let mut drawer =
+            TerminalDrawer::new(io::stdout(), self.image_printer.clone(), self.options.font_size_fallback)?;
         loop {
             self.render(&mut drawer)?;
             self.update_widgets(&mut drawer)?;
@@ -117,6 +124,14 @@ impl<'a> Presenter<'a> {
     fn render(&mut self, drawer: &mut TerminalDrawer<Stdout>) -> RenderResult {
         let result = match &self.state {
             PresenterState::Presenting(presentation) => drawer.render_slide(presentation),
+            PresenterState::SlideIndex(presentation) => {
+                drawer.render_slide(presentation)?;
+                drawer.render_slide_index(presentation)
+            }
+            PresenterState::KeyBindings(presentation) => {
+                drawer.render_slide(presentation)?;
+                drawer.render_key_bindings(presentation)
+            }
             PresenterState::Failure { error, .. } => drawer.render_error(error),
             PresenterState::Empty => panic!("cannot render without state"),
         };
@@ -132,7 +147,7 @@ impl<'a> Presenter<'a> {
                 return CommandSideEffect::Reload;
             }
             Command::HardReload => {
-                if matches!(self.mode, PresentMode::Development) {
+                if matches!(self.options.mode, PresentMode::Development) {
                     self.resources.clear();
                 }
                 return CommandSideEffect::Reload;
@@ -142,16 +157,21 @@ impl<'a> Presenter<'a> {
         };
 
         // Now apply the commands that require a presentation.
-        let PresenterState::Presenting(presentation) = &mut self.state else {
-            return CommandSideEffect::None;
+        let presentation = match &mut self.state {
+            PresenterState::Presenting(presentation)
+            | PresenterState::SlideIndex(presentation)
+            | PresenterState::KeyBindings(presentation) => presentation,
+            _ => {
+                return CommandSideEffect::None;
+            }
         };
         let needs_redraw = match command {
             Command::Redraw => true,
-            Command::JumpNextSlide => presentation.jump_next_slide(),
-            Command::JumpPreviousSlide => presentation.jump_previous_slide(),
-            Command::JumpFirstSlide => presentation.jump_first_slide(),
-            Command::JumpLastSlide => presentation.jump_last_slide(),
-            Command::JumpSlide(number) => presentation.jump_slide(number.saturating_sub(1) as usize),
+            Command::Next => presentation.jump_next(),
+            Command::Previous => presentation.jump_previous(),
+            Command::FirstSlide => presentation.jump_first_slide(),
+            Command::LastSlide => presentation.jump_last_slide(),
+            Command::GoToSlide(number) => presentation.go_to_slide(number.saturating_sub(1) as usize),
             Command::RenderWidgets => {
                 if presentation.render_slide_widgets() {
                     self.slides_with_pending_widgets.insert(self.state.presentation().current_slide_index());
@@ -160,14 +180,29 @@ impl<'a> Presenter<'a> {
                     return CommandSideEffect::None;
                 }
             }
+            Command::ToggleSlideIndex => {
+                self.toggle_slide_index();
+                true
+            }
+            Command::ToggleKeyBindingsConfig => {
+                self.toggle_key_bindings();
+                true
+            }
+            Command::CloseModal => {
+                let presentation = mem::take(&mut self.state).into_presentation();
+                self.state = PresenterState::Presenting(presentation);
+                true
+            }
             // These are handled above as they don't require the presentation
-            Command::Reload | Command::HardReload | Command::Exit => panic!("unreachable commands"),
+            Command::Reload | Command::HardReload | Command::Exit => {
+                panic!("unreachable commands")
+            }
         };
         if needs_redraw { CommandSideEffect::Redraw } else { CommandSideEffect::None }
     }
 
     fn try_reload(&mut self, path: &Path) {
-        if matches!(self.mode, PresentMode::Presentation) {
+        if matches!(self.options.mode, PresentMode::Presentation) {
             return;
         }
         self.slides_with_pending_widgets.clear();
@@ -175,10 +210,10 @@ impl<'a> Presenter<'a> {
             Ok(mut presentation) => {
                 let current = self.state.presentation();
                 if let Some(modification) = PresentationDiffer::find_first_modification(current, &presentation) {
-                    presentation.jump_slide(modification.slide_index);
+                    presentation.go_to_slide(modification.slide_index);
                     presentation.jump_chunk(modification.chunk_index);
                 } else {
-                    presentation.jump_slide(current.current_slide_index());
+                    presentation.go_to_slide(current.current_slide_index());
                     presentation.jump_chunk(current.current_chunk());
                 }
                 self.state = PresenterState::Presenting(presentation)
@@ -193,14 +228,16 @@ impl<'a> Presenter<'a> {
     fn load_presentation(&mut self, path: &Path) -> Result<Presentation, LoadPresentationError> {
         let content = fs::read_to_string(path).map_err(LoadPresentationError::Reading)?;
         let elements = self.parser.parse(&content)?;
-        let export_mode = matches!(self.mode, PresentMode::Export);
+        let export_mode = matches!(self.options.mode, PresentMode::Export);
         let mut presentation = PresentationBuilder::new(
             CodeHighlighter::default(),
             self.default_theme,
             &mut self.resources,
             &mut self.typst,
             &self.themes,
-            self.builder_options.clone(),
+            ImageRegistry(self.image_printer.clone()),
+            self.options.bindings.clone(),
+            self.options.builder_options.clone(),
         )
         .build(elements)?;
         if export_mode {
@@ -209,6 +246,28 @@ impl<'a> Presenter<'a> {
 
         Ok(presentation)
     }
+
+    fn toggle_slide_index(&mut self) {
+        let state = mem::take(&mut self.state);
+        match state {
+            PresenterState::Presenting(presentation) | PresenterState::KeyBindings(presentation) => {
+                self.state = PresenterState::SlideIndex(presentation)
+            }
+            PresenterState::SlideIndex(presentation) => self.state = PresenterState::Presenting(presentation),
+            other => self.state = other,
+        }
+    }
+
+    fn toggle_key_bindings(&mut self) {
+        let state = mem::take(&mut self.state);
+        match state {
+            PresenterState::Presenting(presentation) | PresenterState::SlideIndex(presentation) => {
+                self.state = PresenterState::KeyBindings(presentation)
+            }
+            PresenterState::KeyBindings(presentation) => self.state = PresenterState::Presenting(presentation),
+            other => self.state = other,
+        }
+    }
 }
 
 enum CommandSideEffect {
@@ -224,6 +283,8 @@ enum PresenterState {
     #[default]
     Empty,
     Presenting(Presentation),
+    SlideIndex(Presentation),
+    KeyBindings(Presentation),
     Failure {
         error: String,
         presentation: Presentation,
@@ -233,24 +294,30 @@ enum PresenterState {
 impl PresenterState {
     fn presentation(&self) -> &Presentation {
         match self {
-            Self::Presenting(presentation) => presentation,
-            Self::Failure { presentation, .. } => presentation,
+            Self::Presenting(presentation)
+            | Self::SlideIndex(presentation)
+            | Self::KeyBindings(presentation)
+            | Self::Failure { presentation, .. } => presentation,
             Self::Empty => panic!("state is empty"),
         }
     }
 
     fn presentation_mut(&mut self) -> &mut Presentation {
         match self {
-            Self::Presenting(presentation) => presentation,
-            Self::Failure { presentation, .. } => presentation,
+            Self::Presenting(presentation)
+            | Self::SlideIndex(presentation)
+            | Self::KeyBindings(presentation)
+            | Self::Failure { presentation, .. } => presentation,
             Self::Empty => panic!("state is empty"),
         }
     }
 
     fn into_presentation(self) -> Presentation {
         match self {
-            Self::Presenting(presentation) => presentation,
-            Self::Failure { presentation, .. } => presentation,
+            Self::Presenting(presentation)
+            | Self::SlideIndex(presentation)
+            | Self::KeyBindings(presentation)
+            | Self::Failure { presentation, .. } => presentation,
             Self::Empty => panic!("state is empty"),
         }
     }
