@@ -1,14 +1,21 @@
 use super::{
     draw::{RenderError, RenderResult},
     layout::Layout,
-    media::{GraphicsMode, Image, MediaRender},
     properties::CursorPosition,
     terminal::Terminal,
     text::TextDrawer,
 };
 use crate::{
-    markdown::text::WeightedLine,
-    presentation::{AsRenderOperations, MarginProperties, PreformattedLine, RenderOnDemand, RenderOperation},
+    markdown::text::WeightedTextBlock,
+    media::{
+        image::Image,
+        printer::{PrintOptions, ResourceProperties},
+        scale::scale_image,
+    },
+    presentation::{
+        AsRenderOperations, ImageProperties, ImageSize, MarginProperties, PreformattedLine, RenderOnDemand,
+        RenderOperation,
+    },
     render::{layout::Positioning, properties::WindowSize},
     style::Colors,
     theme::Alignment,
@@ -24,35 +31,26 @@ where
     colors: Colors,
     max_modified_row: u16,
     layout: LayoutState,
-    graphics_mode: GraphicsMode,
 }
 
 impl<'a, W> RenderEngine<'a, W>
 where
     W: io::Write,
 {
-    pub(crate) fn new(
-        terminal: &'a mut Terminal<W>,
-        window_dimensions: WindowSize,
-        graphics_mode: GraphicsMode,
-    ) -> Self {
+    pub(crate) fn new(terminal: &'a mut Terminal<W>, window_dimensions: WindowSize) -> Self {
         let max_modified_row = terminal.cursor_row;
         let current_rect = WindowRect { dimensions: window_dimensions, start_column: 0 };
         let window_rects = vec![current_rect.clone()];
-        Self {
-            terminal,
-            window_rects,
-            colors: Default::default(),
-            max_modified_row,
-            layout: Default::default(),
-            graphics_mode,
-        }
+        Self { terminal, window_rects, colors: Default::default(), max_modified_row, layout: Default::default() }
     }
 
     pub(crate) fn render<'b>(mut self, operations: impl Iterator<Item = &'b RenderOperation>) -> RenderResult {
+        self.terminal.begin_update()?;
         for operation in operations {
             self.render_one(operation)?;
         }
+        self.terminal.end_update()?;
+        self.terminal.flush()?;
         Ok(())
     }
 
@@ -63,10 +61,11 @@ where
             RenderOperation::PopMargin => self.pop_margin(),
             RenderOperation::SetColors(colors) => self.set_colors(colors),
             RenderOperation::JumpToVerticalCenter => self.jump_to_vertical_center(),
+            RenderOperation::JumpToRow { index } => self.jump_to_row(*index),
             RenderOperation::JumpToBottomRow { index } => self.jump_to_bottom(*index),
-            RenderOperation::RenderText { line: texts, alignment } => self.render_text(texts, alignment),
+            RenderOperation::RenderText { line, alignment } => self.render_text(line, alignment),
             RenderOperation::RenderLineBreak => self.render_line_break(),
-            RenderOperation::RenderImage(image) => self.render_image(image),
+            RenderOperation::RenderImage(image, properties) => self.render_image(image, properties),
             RenderOperation::RenderPreformattedLine(operation) => self.render_preformatted_line(operation),
             RenderOperation::RenderDynamic(generator) => self.render_dynamic(generator.as_ref()),
             RenderOperation::RenderOnDemand(generator) => self.render_on_demand(generator.as_ref()),
@@ -127,13 +126,18 @@ where
         Ok(())
     }
 
+    fn jump_to_row(&mut self, index: u16) -> RenderResult {
+        self.terminal.move_to_row(index)?;
+        Ok(())
+    }
+
     fn jump_to_bottom(&mut self, index: u16) -> RenderResult {
         let target_row = self.current_dimensions().rows.saturating_sub(index).saturating_sub(1);
         self.terminal.move_to_row(target_row)?;
         Ok(())
     }
 
-    fn render_text(&mut self, text: &WeightedLine, alignment: &Alignment) -> RenderResult {
+    fn render_text(&mut self, text: &WeightedTextBlock, alignment: &Alignment) -> RenderResult {
         let layout = self.build_layout(alignment.clone());
         let text_drawer = TextDrawer::new(&layout, text, self.current_dimensions(), &self.colors)?;
         text_drawer.draw(self.terminal)
@@ -144,13 +148,33 @@ where
         Ok(())
     }
 
-    fn render_image(&mut self, image: &Image) -> RenderResult {
-        let position = CursorPosition { row: self.terminal.cursor_row, column: self.current_rect().start_column };
-        let (_, height) = MediaRender::new(self.graphics_mode.clone())
-            .draw_image(image, position, self.current_dimensions())
-            .map_err(|e| RenderError::Other(Box::new(e)))?;
-        let row = self.terminal.cursor_row + height as u16;
-        self.terminal.sync_cursor_row(row);
+    fn render_image(&mut self, image: &Image, properties: &ImageProperties) -> RenderResult {
+        let rect = self.current_rect();
+        let starting_position = CursorPosition { row: self.terminal.cursor_row, column: rect.start_column };
+
+        let (width, height) = image.dimensions();
+        let (cursor_position, columns, rows) = match properties.size {
+            ImageSize::Scaled => {
+                let scale = scale_image(&rect.dimensions, width, height, &starting_position);
+                (CursorPosition { row: starting_position.row, column: scale.start_column }, scale.columns, scale.rows)
+            }
+            ImageSize::Specific(columns, rows) => (starting_position.clone(), columns, rows),
+        };
+
+        let options = PrintOptions {
+            columns,
+            rows,
+            cursor_position,
+            z_index: properties.z_index,
+            column_width: rect.dimensions.pixels_per_column() as u16,
+            row_height: rect.dimensions.pixels_per_row() as u16,
+        };
+        self.terminal.print_image(image, &options)?;
+        if properties.restore_cursor {
+            self.terminal.move_to(starting_position.column, starting_position.row)?;
+        } else {
+            self.terminal.move_to_row(starting_position.row + rows)?;
+        }
         Ok(())
     }
 
@@ -173,7 +197,7 @@ where
         if *unformatted_length as u16 > max_line_length {
             let lines_wrapped = *unformatted_length as u16 / max_line_length;
             let new_row = self.terminal.cursor_row + lines_wrapped;
-            self.terminal.sync_cursor_row(new_row);
+            self.terminal.sync_cursor_row(new_row)?;
         }
 
         // Restore colors
@@ -230,15 +254,13 @@ where
         let new_column_count = (total_column_units - columns[column_index]) * unit_width as u16;
         let new_size = current_rect.dimensions.shrink_columns(new_column_count);
         let mut dimensions = WindowRect { dimensions: new_size, start_column };
-        if columns.len() != 1 {
-            // Shrink every column's right edge except for last
-            if column_index < columns.len() - 1 {
-                dimensions = dimensions.shrink_right(4);
-            }
-            // Shrink every column's left edge except for first
-            if column_index > 0 {
-                dimensions = dimensions.shrink_left(4);
-            }
+        // Shrink every column's right edge except for last
+        if column_index < columns.len() - 1 {
+            dimensions = dimensions.shrink_right(4);
+        }
+        // Shrink every column's left edge except for first
+        if column_index > 0 {
+            dimensions = dimensions.shrink_left(4);
         }
 
         self.window_rects.push(dimensions);
@@ -292,7 +314,7 @@ impl WindowRect {
     }
 
     fn shrink_left(&self, size: u16) -> Self {
-        let dimensions = self.dimensions.clone();
+        let dimensions = self.dimensions.shrink_columns(size);
         let start_column = self.start_column.saturating_add(size);
         Self { dimensions, start_column }
     }
